/**
  *  \file client/widgets/chartwidget.cc
  *  \brief Starchart Widget
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/chartwidget.h"
#include "gfx/palette.h"
#include "game/global.h"
#include "gfx/clip.h"

/** Constructor. Call setCenterPosition() to set position to display. */
WChartWidget::WChartWidget()
    : is_valid(false)
{
    viewport.center = GPoint(2000, 2000);
    viewport.mult = viewport.divi = 1;
    viewport.vec_id = 0;
    viewport.drawing_filter = viewport.NO_FILTER;
}

WChartWidget::~WChartWidget()
{ }

void
WChartWidget::drawContent(GfxCanvas& can)
{
    drawSolidBar(can, getExtent(), standard_colors[COLOR_BLACK]);
    GfxClipRectFilter clip(getExtent(), can);
    if (isValid()) {
        viewport.view = getExtent();
        viewport.init();            // so that drawPre/drawPost can use it
        drawPre(clip);
        viewport.draw(can, getChartOpts(true, viewport.mult, viewport.divi),
                      getDisplayedTurn().getCurrentUniverse(),
                      getDisplayedTurn().getChartBBox());
    }
    drawPost(clip);
    can.update(getExtent());
}

/** Set position of the center of this chart widget. Will update the
    display. This will also mark the widget valid. */
void
WChartWidget::setCenterPosition(GPoint pt)
{
    if (!is_valid || pt != viewport.center) {
        is_valid = true;
        viewport.center = pt;
        drawWidget();
    }
}

/** Set chart widget invalid. This will remove the map display and
    only display a black area. */
void
WChartWidget::setInvalid()
{
    if (is_valid) {
        is_valid = false;
        drawWidget();
    }
}

/** Pre-draw hook. Override in descendant classes to draw something
    below the map. This is only called if the starchart viewport
    actually is valid. */
void
WChartWidget::drawPre(GfxCanvas& /*can*/)
{ }

/** Post-draw hook. Override in descendant classes to draw something
    atop the map. This is also called with invalid starchart viewport,
    so it can draw a notice why the map is not displayed this time. */
void
WChartWidget::drawPost(GfxCanvas& /*can*/)
{ }

/** Zoom in.
    \return true iff zoom factor actually changed. */
bool
WChartWidget::zoomIn()
{
    if (viewport.zoomIn(getDisplayedTurn().getChartBBox())) {
        drawWidget(false);
        return true;
    } else {
        return false;
    }
}

/** Zoom out.
    \return true iff zoom factor actually changed. */
bool
WChartWidget::zoomOut()
{
    if (viewport.zoomOut(getDisplayedTurn().getChartBBox())) {
        drawWidget(false);
        return true;
    } else {
        return false;
    }
}
