/**
  *  \file int/if/shippred.h
  *  \brief Interpreter Game Interface: Ship Prediction
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_SHIPPRED_H
#define PCC_V2_INT_IF_SHIPPRED_H

#include "int/predict.h"
#include "game/ship.h"

/** Interpreter ship predictor. This predicts the effect of script
    commands (from auto tasks) on a ship. */
class IntShipPredictor : public IntAutoTaskPredictor {
    enum { MAX_XYS = 30 };

    GShipTurnPredictor pred;

    GPoint positions[MAX_XYS];
    int num_xys;
    int fuel_xys;
    int turn_fuel;
    bool have_fuel;

    void setWaypoint(IntArgBlock& args);
    void predictTurn();

 public:
    IntShipPredictor(int id);
    ~IntShipPredictor();

    bool predictInstruction(const string_t& name, IntArgBlock& args);

    int getNumPositions() const;
    int getNumFuelPositions() const;
    int getNumTurns() const;
    int getNumFuelTurns() const;
    int getMovementFuel() const;
    int getCloakFuel() const;
    int getRemainingFuel() const;
    string_t getFCode() const;
    int getSpeed() const;

    GPoint getPosition(int index) const;
    GPoint getPosition() const;
};


inline int
IntShipPredictor::getNumPositions() const
{
    return num_xys;
}

inline int
IntShipPredictor::getNumFuelPositions() const
{
    return fuel_xys;
}

inline int
IntShipPredictor::getNumTurns() const
{
    return pred.getTurnCount();
}

inline int
IntShipPredictor::getNumFuelTurns() const
{
    return turn_fuel;
}

inline int
IntShipPredictor::getMovementFuel() const
{
    return pred.getMovementFuelUsed();
}

inline int
IntShipPredictor::getCloakFuel() const
{
    return pred.getCloakFuelUsed();
}

inline int
IntShipPredictor::getRemainingFuel() const
{
    return pred.getFuel();
}

inline string_t
IntShipPredictor::getFCode() const
{
    return pred.getFCode();
}

inline int
IntShipPredictor::getSpeed() const
{
    return pred.getSpeed();
}

inline GPoint
IntShipPredictor::getPosition(int index) const
{
    return positions[index];
}

inline GPoint
IntShipPredictor::getPosition() const
{
    return pred.getPosition();
}

#endif
